/*
	Publimark
 
	Copyright (c) 2004, Gatan Le Guelvouit <leguelvouit@lss.supelec.fr>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

#include "init.h"

#define CARRE(x) ((x)*(x))

vector<double> init::D(const vector<double>& u, const vector<double>& b)
{
	vector<double> r(u.size());
	double n = 0; 
	
	for(uint i=0; i<u.size(); i++)
	{
		n+=CARRE(u[i] - b[i]);
	}
	
	for(uint i=0; i<u.size(); i++)
	{
		r[i]=(u[i] - b[i])/sqrt(n);
	}
	
	return r;
}


seed init::embed(vector<double>& x, const char* pub)
{
	if(x.size()<(CRYPTO_RATE*SEED_BITS) + 2*REGISTER-2) ERR("[init::embed] Wrong number of host data");
	
	seed		seed_tmp;
	tcq		  init_tcq(REGISTER, delta);
	string  L = string(seed_tmp);

	vector<int>    f((CRYPTO_RATE*SEED_BITS) + REGISTER-1, -1); // Pattern to encode the key
	
	LOG("--  [init::embed] Secret temp. seed : " << seed_tmp << endl);
		
	// The encoded key to be transmitted
	L=crypto::encrypt(L, pub); 
		
	for(uint i=0; i<8*L.size(); i++) 
	{
		f[i+REGISTER-1]=((int(L[i/8*sizeof(char)])+128)>>(i%(8*sizeof(char))))%2;
	} 

	// Closest codeword
	vector<double> u = init_tcq.encode(init_tcq.decode(x, f)); 
	
	bool   done   = false;
	double lambda = 0; 
	
	do // Loop to go closer to U, but not too much (Monte Carlo rulez)
	{
		vector<double> tmp = x; 
		for(uint i=0; i<tmp.size(); i++) 
		{
			if(tmp[i]>0) tmp[i]=short(tmp[i]+0.5); else tmp[i]=short(tmp[i]-0.5);
		}
			
		vector<double> b = init_tcq.encode(init_tcq.decode(tmp));
		
		done=(b==u);
		
		if(!done)
		{
			const vector<double> d = D(u, b);
			const double         alpha = delta/10;
			
			for(uint i=0; i<x.size(); i++) x[i]+=alpha*d[i];
		}
	}
	while(!done);	

	return seed_tmp;
}


seed init::extract(vector<double>& x, const char* prv)
{
	if(x.size()<(CRYPTO_RATE*SEED_BITS) + 2*REGISTER-2) ERR("[init::extract] Wrong number of host data");
	
	tcq		         init_tcq(REGISTER, delta);
	seed					 seed_tmp; 
	string         L;
		
	vector<symbole> dec = init_tcq.decode(x); 
	
	for(uint i=0; i<CRYPTO_RATE*SEED_BITS/8; i++)
	{
		int c = 0;
		
		for(uint j=0; j<8*sizeof(char); j++) 
		{
			if(dec[i*8*sizeof(char) + j + REGISTER-1].bit) c+=1<<j;
		} 
		
		L+=char(c-128); 
	} 
	
	L=crypto::decrypt(L, prv); 
		
	return seed(L);
}



