/*
	Publimark
 
	Copyright (c) 2004, Gatan Le Guelvouit <leguelvouit@lss.supelec.fr>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

#include <stdio.h>
#include <fstream>
#include <audiofile.h>

#include "scs.h"
#include "init.h"
#include "crypto.h"
#include "arguments.h"

#define CARRE(x) ((x)*(x))


#include "send.cpp"
#include "receive.cpp"

int main(const int argc, const char** argv)
{
	Arguments A(argc, argv);
	
	string   key; 
	double   power;
	
	A.get("--key", key, "");
	A.get("--pow", power, 3);
	
	if(A.has("-h|--help") || (argc<3 && !A.has("-K")))
  {
    cout << "PUBLIMARK Steganographic tool " << VERSION << " - G. Le Guelvouit\n";
    cout << "LSS/Supelec                                  May 2004\n";
    cout << "leguelvouit@lss.supelec.fr         www.lss.supelec.fr\n\n";
		
    cout << "Usage: cat my_secret.txt | " << argv[0] << " input_file output_file.{aif|wav} --key=public_file\n";
		cout << "       " << argv[0] << " input_file --key=private_file\n";
		cout << "       " << argv[0] << " -K public_file private_file \n\n";

    return 0;
  }
	
	Botan::LibraryInitializer botan_init;
	
	// Key generation 
	if(A.has("-K"))
	{
		string	 pub, prv;
		
		crypto::get_pair(pub, prv);
		LOG("--[send::main] Keys generation OK.\n");
		
		if(argc==4)
		{
			LOG("--[send::main] Public file : " << argv[2] << "   Private file : " << argv[3] << endl);
			
			TRACE; ofstream public_file(argv[2]); TOK;
			TRACE; ofstream private_file(argv[3]); TOK;
			
			if(!public_file)  {cerr << "ERROR: Could not write " << argv[2] << ".\n" ; return 1;}
			if(!private_file) {cerr << "ERROR: Could not write " << argv[3] << ".\n" ; return 1;}
			
			public_file  << "# A Publimark ver. " << VERSION << " public key\n"  << pub;
			private_file << "# A Publimark ver. " << VERSION << " private key\n" << prv;

			public_file.close();
			private_file.close();
		}
		else 
		{
			cout << "\n# A Publimark ver. " << VERSION << " public key\n" << pub;
			cout << "\n# A Publimark ver. " << VERSION << " private key\n"<< prv << endl;
		}
		
		return 0;
	}
	else if(argc==3) return receive(argv[1], key.c_str(), power); else return send(argv[1], argv[2], key.c_str(), power);
}


