/*
	Publimark
 
	Copyright (c) 2004, Gatan Le Guelvouit <leguelvouit@lss.supelec.fr>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

int receive(const char* input, const char* key, const double power)
{
	// Opening input cover file and getting infos
	AFfilehandle infile       = afOpenFile(input, "r", NULL); 
	
	if(infile ==NULL) ERR("Cannot open audio file " << input);
	
	int  af_format, af_width;
	seed Stmp; 
	
	afGetSampleFormat(infile, AF_DEFAULT_TRACK, &af_format, &af_width);
	
	int    af_channel		= afGetChannels(infile, AF_DEFAULT_TRACK);
	int    af_frame     = afGetFrameCount(infile, AF_DEFAULT_TRACK); 
	double af_rate      = afGetRate(infile, AF_DEFAULT_TRACK);
	
	{ // Initialization 
		int    nb_data  = CRYPTO_RATE*SEED_BITS + 2*REGISTER-2;
		int    nb_frame = (INIT_GAP*(CRYPTO_RATE*SEED_BITS+2*REGISTER-2)+1)/af_channel;
		short* buffer   = new short[af_channel*nb_frame];
	
		vector<double> data(nb_data);

		// Reading host data
		afReadFrames(infile, AF_DEFAULT_TRACK, (void*)buffer, nb_frame);
		for(uint i=0; i<data.size(); i++) data[i]=double(buffer[INIT_GAP*i]);
		
		init   my_init(pow(10.0, power/10));
		
		Stmp=my_init.extract(data, key); LOG("--  [main::receive] Secret temp. seed : " << Stmp << endl);
	
		af_frame-=nb_frame; 
		
		LOG("--[main::receive] Initialisation phase OK.\n");
	}	

	{ // Permanent phase
		int    c;
		int    cc = 0;
		
		Stmp.init();
		
		do
		{
			int    nb_data  = 8*sizeof(char);
			int    nb_frame = PERM_GAP*nb_data/af_channel;
			short* buffer   = new short[af_channel*nb_frame];
			
			vector<double> data(nb_data);

			// Reading host data
			afReadFrames(infile, AF_DEFAULT_TRACK, (void*)buffer, nb_frame);
			for(uint i=0; i<data.size(); i++) 
			{
				data[i]=double(buffer[PERM_GAP*i]);
			} 
			
			scs my_scs(Stmp, pow(10.0, power/10));
			
			c=my_scs.extractc(data, false); 
			if(char(c)!=char(EOF)) cout << char(c);
	
			af_frame-=nb_frame;
			
		} while(char(c)!=char(EOF) && af_frame*af_channel>=sizeof(char)); 
		
		LOG("--[main::receive] Permanent phase OK.\n");
	}
	
	afCloseFile(infile);
		
	return 0;
}

