/*
	Publimark
 
	Copyright (c) 2004, Gatan Le Guelvouit <leguelvouit@lss.supelec.fr>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

#include "scs.h"

#define CARRE(x) ((x)*(x))

void scs::embed(vector<double>& x, const char c, const bool reset)
{
	if(x.size()<8*sizeof(char)) ERR("[scs::embed] Wrong number of host data");
	
	if(reset) scs_seed.init();
	
	const double alpha = sqrt(CARRE(delta) - 2.71)/delta;
	
	for(uint i=0; i<8*sizeof(c); i++)
	{
		double m    = (double(((int(c)+128)>>i)%2)!=0);
		double u, T = delta*(scs_seed.get_random() - 0.5 + 0.5*m);
		
		if(x[i]>0) u=delta*int((x[i] - T)/delta + 0.5)+T; else u=delta*int((x[i] - T)/delta - 0.5)+T;
	
		x[i]+=alpha*(u - x[i]);
	}
}

void scs::embed(vector<double>& x, const string& s, const bool reset)
{
	if(x.size()<s.size()*8*sizeof(char)) ERR("[scs::embed] Wrong number of host data");

	if(reset) scs_seed.init();
	
	for(uint i=0; i<s.size(); i++)
	{
		vector<double> tmp(8*sizeof(char));
		
		for(uint j=0; j<8*sizeof(char); j++) tmp[j]=x[i*8*sizeof(char) + j];
		
		embed(tmp, s[i], false);
		
		for(uint j=0; j<8*sizeof(char); j++) x[i*8*sizeof(char) + j]=tmp[j];
	}
}


char scs::extractc(const vector<double>& x, const bool reset)
{
	if(x.size()<8*sizeof(char)) ERR("[scs::extract] Wrong number of host data");

	if(reset) scs_seed.init();
	
	int c = 0;
	
	for(uint i=0; i<8*sizeof(char); i++)
	{
		double u0, T0 = delta*(scs_seed.get_random() - 0.5);
		double u1, T1 = T0 + 0.5*delta;
		
		if(x[i]>0)
		{
			u0=T0+delta*int((x[i] - T0)/delta + 0.5); 
			u1=T1+delta*int((x[i] - T1)/delta + 0.5); 
		}
		else 
		{
			u0=T0+delta*int((x[i] - T0)/delta - 0.5);
			u1=T1+delta*int((x[i] - T1)/delta - 0.5);
		}
		
		if(CARRE(u0 - x[i]) > CARRE(u1 - x[i])) c+=(1<<i);
	}
	
	return char(c-128);
}


string scs::extracts(const vector<double>& x, const bool reset)
{
	if(x.size()<sizeof(char)) ERR("[scs::extract] Wrong number of host data");
	
	if(reset) scs_seed.init();
	
	string s;

	for(uint i=0; i<x.size()/(8*sizeof(char)); i++)
	{
		vector<double> tmp(8*sizeof(char));
		
		for(uint j=0; j<8*sizeof(char); j++) tmp[j]=x[i*8*sizeof(char) + j];
		
		s+=extractc(tmp, false);
	}
	
	return s;
}



