/*
	Publimark
 
	Copyright (c) 2004, Gatan Le Guelvouit <leguelvouit@lss.supelec.fr>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

#include "tcq.h"

#define DS 0 

#define CARRE(a)  ((a)*(a))
#define P2(a)     (1<<(a))



int tcq::g_index(const double T, const double x)
{		
	const double X = x - T;
	
	if(X>0) return int(X/delta + 0.5); else return int(X/delta - 0.5);
}

double tcq::g_u(const double T, const int index)
{	
	return delta*index + T;
}

vector<double> tcq::encode(const vector<symbole>& m)
{
	vector<double> x;
	uint					 state = 0;
	
	for(uint i=0; i<m.size(); i++)
	{
		const double quant = g_output(state, m[i].bit, i);	
		const double xq    = g_u(quant, m[i].index);
		
		x.push_back(xq);
		
		state=g_next_state(state, m[i].bit);
	}
	
	return x;
}

double tcq::g_output(const uint state, const bool bit, const uint t)
{
	srandom(t*P2(d) + 2*state + bit);
	
	return (double(random()%999999)/999999 - 0.5)*delta;
}


/*!
  \param x Host signal.
  \param f Constraint values (0 or 1 for a constraint, -1 if not).
  \return Decoded message.
*/
vector<symbole> tcq::decode(const vector<double>& x, const vector<int>& f)
{
	const uint nbs     = P2(d-1);             // Nombre d'tats
  const uint d_state = int(DS)*(P2(d-1)-1); // Etat de dpart et de fin

  // Treillis structures
  vector<path>          paths(nbs, path(x.size()));
  vector<vector<char> > todo(P2(d-1), vector<char>(x.size()+1, 0));
  
  todo[d_state][0]=1; // First state

  for(unsigned int t=0; t<x.size(); t++) // Samples loop
  {
    uint bd = 0; 
    uint bf = 2;

    if(t<x.size()-d+1) // General case
    {
      if(f.size()==x.size()-d+1)
      {
				if(f[t]==0) {bd=0; bf=1;}
				if(f[t]==1) {bd=1; bf=2;}
      }
    }
    else // "Bourrage" bits (what is the english word?)  
    {
      if(DS==1) {bd=1; bf=2;} else {bd=0; bf=1;}  
    }

    for(uint s=0; s<nbs; s++)
    {  
      if(todo[s][t]==1)
      {      
				for(uint b=bd; b<bf; b++)
				{
					const uint   b_state   = g_next_state(s, b); 
					const double quant		 = g_output(s, b, t);
					const int    index     = g_index(quant, x[t]);
					double		   b_err     = distance(x[t], g_u(quant, index));
  
					if(paths[s].err+b_err < paths[b_state].score) // This path is better than the current one 
					{
						paths[b_state].score=paths[s].err+b_err;
						for(uint i=0; i<t; i++) paths[b_state].propose[i]=paths[s].mess[i]; 
						
						paths[b_state].propose[t].bit  =bool(b);
						paths[b_state].propose[t].index=index;
					}

					todo[b_state][t+1]=1; // Which states for the next time 
				}
      }
    }

    for(uint s=0; s<nbs; s++) // We keep only the bests
    {
      if(todo[s][t+1]==1)
      {
				paths[s].err=paths[s].score;
				paths[s].score=+DBL_MAX; 
				paths[s].mess=paths[s].propose; 
      }
    }
  }
	//LOG("--    [tcq::decode] D = " << paths[d_state].err/x.size() << endl);
	
	return paths[d_state].mess;
}
