/*
	Publimark
 
	Copyright (c) 2004, Gatan Le Guelvouit <leguelvouit@lss.supelec.fr>
 
	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License as
	published by the Free Software Foundation; either version 2 of
	the License, or (at your option) any later version.
 
	This program is distributed in the hope that it will be
	useful, but WITHOUT ANY WARRANTY; without even the implied
	warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
	PURPOSE.  See the GNU General Public License for more details.
 
	You should have received a copy of the GNU General Public
	License along with this program; if not, write to the Free
	Software Foundation, Inc., 59 Temple Place, Suite 330, Boston,
	MA 02111-1307, USA.
*/

#ifndef __TCQ_H__
#define __TCQ_H__

#include <math.h>
#include <vector>
#include <float.h>


#include "common.h"

using namespace std;

class symbole
{
public:
	bool bit;
	int  index;
	
	symbole() : bit(false), index(0) {};
};

//! A class for message coding using treillis coded quantization.
class tcq
{
private:
	uint   d;     // Register length
	double delta; // Quantifization step
	
protected:
  //! Give next state considering current state and a given bit.
  inline uint g_next_state(const uint state, const bool bit);

  //! Which quantization must be used.
  double g_output(const uint state, const bool bit, const uint t);

	int  g_index(const double T, const double x);
	
	double g_u(const double T, const int index);
	
	double distance(const double x, const double y) {return (x-y)*(x-y);};
	
public:
	tcq(const uint n_d, const double n_delta) : d(n_d), delta(n_delta) {};		
		
  //! Encode a message.
  vector<double> encode(const vector<symbole>& m);

  //! Decode a signal.
  vector<symbole> decode(const vector<double>& x, const vector<int>& f=vector<int>(0));
};

/*!
  \param state Current treillis state.
  \param bit Bit to encode.
  \return New state.
*/
inline uint tcq::g_next_state(const uint state, const bool bit) 
{
  if(bit) return (state>>1)+(1<<(d-2)); else return state>>1;
}


//! A treillis path.
class path
{
public:
  double					err, score;
  vector<symbole> mess, propose;

  path(uint n) : err(0), score(+DBL_MAX), mess(n), propose(n) {};
};

#endif
